#include "FS.h"
#include "SPIFFS.h"

/* You only need to format SPIFFS the first time you run a
   test or else use the SPIFFS plugin to create a partition
   https://github.com/me-no-dev/arduino-esp32fs-plugin */



void readFile(fs::FS &fs, const char * path){
    Serial.printf("Reading file: %s\r\n", path);

    File file = fs.open(path);
    if(!file || file.isDirectory()){
        Serial.println("- failed to open file for reading");
        return;
    }

    Serial.println("- read from file:");
    while(file.available()){
        Serial.write(file.read());
    }
    file.close();
}

void writeFile(fs::FS &fs, const char * path, const char * message){
    Serial.printf("Writing file: %s\r\n", path);

    File file = fs.open(path, FILE_WRITE);
    if(!file){
        Serial.println("- failed to open file for writing");
        return;
    }
    if(file.print(message)){
        Serial.println("- file written");
    } else {
        Serial.println("- write failed");
    }
    file.close();
}

void appendFile(fs::FS &fs, const char * path, const char * message){
    Serial.printf("Appending to file: %s\r\n", path);

    File file = fs.open(path, FILE_APPEND);
    if(!file){
        Serial.println("- failed to open file for appending");
        return;
    }
    if(file.print(message)){
        Serial.println("- message appended");
    } else {
        Serial.println("- append failed");
    }
    file.close();
}

void deleteFile(fs::FS &fs, const char * path){
    Serial.printf("Deleting file: %s\r\n", path);
    if(fs.remove(path)){
        Serial.println("- file deleted");
    } else {
        Serial.println("- delete failed");
    }
}

String readLine (fs::FS &fs, String fileName, long lineNumber)
{
  //if (lineNumber <= numLinesSD (fileName) && lineNumber > 0)
  if (lineNumber > 0)
  {
    File myFile = fs.open(fileName);

    if (myFile) 
    {
      String readLine = "";
      //Serial.println(".txt:");

      int counterLine = 0;
      char lastChar;
      while (myFile.available()) 
      {
        char currentChar = myFile.read();

        if (lineNumber > 1)  
        {
          if (currentChar == '\n' && lastChar == '\r')
            ++counterLine;
  
          if (counterLine == lineNumber-1)
          {
            if (currentChar != '\n' && currentChar != '\r')
              readLine += currentChar;
          }
          else if (counterLine > lineNumber-1)
            break;
    
          lastChar = currentChar;
        }
        else
        {
          if (currentChar != '\n' && currentChar != '\r')
              readLine += currentChar;

          if (currentChar == '\n' && lastChar == '\r')
            break;
  
          lastChar = currentChar;
        }
      }
  
      myFile.close();

      readLine.replace("\n", "");
      readLine.replace("\r", "");

      return (readLine);
    }
    else
    {
      return ("null");
    }
  }
  else
  {
    return ("null");
  }  
}

void settingsRead(){
  byte linha = 1;

  ssid = readLine (SPIFFS, "/set.txt", linha);
  ++linha;
  
  password = readLine (SPIFFS, "/set.txt", linha);
  ++linha;

  device = readLine (SPIFFS, "/set.txt", linha);
  ++linha;

  delay(200);

  Serial.println("Dentro do settings read");
  Serial.print("ssid: ");
  Serial.println(ssid);
  Serial.print("password: ");
  Serial.println(password);
  Serial.print("device: ");
  Serial.println(device);
}
