#include "secrets.h"
#include <WiFiClientSecure.h>
#include <PubSubClient.h>
#include <ArduinoJson.h>
#include "WiFi.h"
#include "ESPAsyncWebServer.h"
#include <OneWire.h>
#include <DallasTemperature.h>

#include "FS.h"
#include "SPIFFS.h" 

#define FORMAT_SPIFFS_IF_FAILED true

#define phPin 39
#define turbidezPin 36
#define fluxoPin 34
#define temperaturaPin 23

#define modePin 33

OneWire oneWire(temperaturaPin);
DallasTemperature valorTemperatura (&oneWire);

String ssid = "";
String password = "";
String device = "";

unsigned int wifiStatus = 0;
unsigned int gsmStatus = 0;

float consumoAgua = 0; 
float consumoEnergia = 0; 
float cor = 0; 
float condutividade = 0; 
float temperatura = 0; 
float oxigenio = 0; 
float cloro = 0;

String nome = "testeSenai6";

float aux = 0;

int pulsosFluxo = 0;
unsigned long tempoArmazenadoFluxo = 0;

float fluxo = 0;
float pH = 0;
float turbidez = 0;

float h ;
float t;

byte buttonFlag = 0;
byte readFlag = 0;

unsigned int contadorCiclos = 0;

void IRAM_ATTR contaPulsosFluxo(){
  pulsosFluxo++;
}

void IRAM_ATTR modePinInt(){
  serverInit();
}

void serverInit(){
    Serial.println("interrompiiiiii");
    delay(500);
    //disableWiFi();
  
//  buttonFlag = 1;
  
//  if (buttonFlag == 1){
//    buttonFlag = 0;
    //Serial.println("APERTEI O BOTAAAAAAAAAAAAO");
//    disableWiFi ();
//    stopGSM();
//    delay(1000);
//    serverStart();
//    delay (1000);
//    readFlag = 1;  
  }

float leituraFluxo(){
  Serial.println("-------------------Fluxo-------------------");
  if ((millis() - tempoArmazenadoFluxo) >= 1000){
    float litrosSegundo = pulsosFluxo * 0.0022; // 1 litro = 450 pulsos (datasheet no final)
    fluxo = (60 * litrosSegundo);
    pulsosFluxo = 0;
    tempoArmazenadoFluxo = millis();
    Serial.print("Volume: ");
    Serial.println(fluxo);

    fluxo = round(fluxo);
    return fluxo;
  }
}

float leituraPH(){
  Serial.println("---------------------pH--------------------");
  float arrayPH[10];
  float somaPH = 0;
  float mediaPH = 0;
  Serial.print("arrayPH: ");
  for (int i=0; i<=9; i++){
    arrayPH[i] = analogRead(phPin);
    Serial.print(arrayPH[i]);
    Serial.print("\t");
  }
  Serial.println();
  for (int i=0; i<=9; i++){
   somaPH += arrayPH[i]; 
  }

  mediaPH = (somaPH/10);

  float valorPH = ((3.3 * mediaPH)/4095);
  float conversaoPH = (valorPH * 1.493);
  
  pH = (14*conversaoPH/5);
  Serial.println(mediaPH);
  Serial.print("pH: ");
  Serial.println(pH);

  pH = round(pH);
  return pH;
}

float leituraTurbidez() {
  Serial.println("-------------------Turbidez-----------------");
  float arrayTurbidez[10];
  float somaTurbidez = 0;
  float mediaTurbidez = 0;
  for (int i = 0; i <= 9; i++) {
    arrayTurbidez[i] = analogRead(turbidezPin);
  }
  for (int i = 0; i <= 9; i++) {//                    ordenacao dos elementos
    for (int j = (i + 1); j < 10; j++) {
      if (arrayTurbidez[i] > arrayTurbidez[j]) {
        aux = arrayTurbidez[i];
        arrayTurbidez[i] = arrayTurbidez[j];
        arrayTurbidez[j] = aux;
      }
    }
  }
  Serial.print("arrayTurbidez: ");
  for (int i=0; i<=9; i++){
    Serial.print(arrayTurbidez[i]);
    Serial.print("\t");
  }
  Serial.println();
  
  for (int i=1; i<=8; i++){//                         media com eliminacao do maior e menor
    somaTurbidez += arrayTurbidez[i];
    mediaTurbidez = somaTurbidez/8;
  }
  float valorTurbidez = ((3.3 * (mediaTurbidez)) / 4095) + 0.2; //                   conversao de bits para tensao
  float conversaoTurbidez = 1.493 * valorTurbidez; //                                 conversao de tensao 3V3 para 5V
  if (conversaoTurbidez < 2.5) {
    turbidez = 3000;
  }
  else if (conversaoTurbidez > 4.2) {
    turbidez = 0;
  }
  else {
    turbidez = (-1120.4 * (sq(conversaoTurbidez))) + (5742.3 * conversaoTurbidez) - 4353.8; //  conversao para ntu
  }
  Serial.print("Tensao turbidez: ");
  Serial.println(conversaoTurbidez);
  Serial.print("Turbidez: ");
  Serial.println(turbidez);

  turbidez = round(turbidez);
  return turbidez;
}

float leituraTemperatura(){
  Serial.println("------------------Temperatura---------------");
  valorTemperatura.requestTemperatures();
  temperatura = (valorTemperatura.getTempCByIndex(0));
  Serial.print("Temperatura: ");
  Serial.println(temperatura);
  
  temperatura = round(temperatura);
  return temperatura;
  
}

void setup()
{
  pinMode(fluxoPin,INPUT);
  pinMode(phPin, INPUT);
  pinMode(turbidezPin, INPUT);
  pinMode(temperaturaPin, INPUT_PULLUP);
  pinMode(modePin, INPUT_PULLUP);

  valorTemperatura.begin();

  pinMode(temperaturaPin, INPUT_PULLUP);

  if(!SPIFFS.begin(FORMAT_SPIFFS_IF_FAILED)){
        Serial.println("SPIFFS Mount Failed");
        return;
    }

  Serial.begin(115200);

  settingsRead();

  Serial.println("Dentro do setup");
  Serial.print("ssid: ");
  Serial.println(ssid);
  Serial.print("password: ");
  Serial.println(password);
  Serial.print("device: ");
  Serial.println(device);
  
  attachInterrupt(fluxoPin, contaPulsosFluxo, RISING);
  attachInterrupt(modePin, modePinInt, FALLING);

//  Serial.print(digitalRead(33));
//  delay(500);
//  Serial.print(digitalRead(33));
//  delay(500);
//  Serial.println(digitalRead(33));
//  delay(500);

  connectAWS();
  if (wifiStatus == 1){
    Serial.println("Wifi deu ruim");
    //ESP.restart();
    //initGsm();
    //sendData();
  }
  if (gsmStatus == 1){Serial.println("Deu ruim tdo");}
}

void loop()
{
  if (wifiStatus == 0){loopWifi();}
  else if (gsmStatus == 0 ){sendData();}
  else {
    Serial.println("Deu tudo ruim");
    //ESP.restart();
    serverStart();
    }

    contadorCiclos++;
    if (contadorCiclos >= 30){
      Serial.println(contadorCiclos);
      contadorCiclos = 0;
      ESP.restart();
    }
  
  delay(2000);
}/*
https://html.alldatasheetpt.com/html-pdf/1221259/ETC1/YF-S201/229/2/YF-S201.html
*/
